--[[
	Cast Timer 1.1
		Adds a timer (in seconds) to the casting bar.

	By: Moof (Jim Drey, moof@videogamemaps.net) - www.videogamemaps.net
	reworked by exi (exi@stc2.ath.cx)
	added moveable textsupport

	Just like the short desc says, short sweet and simple.

	$Id$
	$Rev$
	$LastChangedBy$
	$Date$
]]--

CastTime = { };
CastTime.delaySum = 0;
CastTime.endTime = 0;
CastTime.startTime = 0;
CastTime.sign = "+";
CastTime.print = 0;
CastTime.display = 1;
CastTime.delay = 1;
CastTime.hun = 1;
CastTime.flock = 0;
CastTime.mf = 0;

function CastTime_OnLoad()
	--Register events and all that good stuff.
	this:RegisterEvent("SPELLCAST_START");
	this:RegisterEvent("SPELLCAST_STOP");
	this:RegisterEvent("SPELLCAST_FAILED");
	this:RegisterEvent("SPELLCAST_INTERRUPTED");
	this:RegisterEvent("SPELLCAST_DELAYED");
	this:RegisterEvent("SPELLCAST_CHANNEL_START");
	this:RegisterEvent("SPELLCAST_CHANNEL_UPDATE");
	this:RegisterEvent("VARIABLES_LOADED");
	CastTimeFrame:Hide();
	if (CastTime_Register) then
		CastTime_Register();
	end	
end

function CastTime_OnEvent()
	--If beginning a spell, show the frame and reset the delaycount.
	if (event == "SPELLCAST_START") then
		CastTime.delaySum = 0;
		CastTimeFrame:Show();
	elseif (event == "SPELLCAST_CHANNEL_START") then
		CastTime.delaySum = 0;
		CastTime.endTime = CastingBarFrame.endTime;
		CastTime.startTime = CastingBarFrame.startTime;
		CastTimeFrame:Show();
	elseif (event == "SPELLCAST_DELAYED") then
		CastTime.delaySum = CastTime.delaySum + arg1;
	elseif (event == "SPELLCAST_CHANNEL_UPDATE") then
		if ( arg1 == 0 ) then
			CastTime.delaySum = 0;
			CastTimeFrame:Hide();
			CastTimeText:SetText("");
			CastTimeMFText:SetText("");
		else
			local origDuration = CastTime.endTime - CastTime.startTime;
			local elapsedTime = GetTime() - CastTime.startTime;
			local losttime = origDuration*1000 - elapsedTime*1000 - arg1;
			CastTime.delaySum = CastTime.delaySum + losttime;
			CastTime.endTime = CastingBarFrame.endTime;
			CastTime.startTime = CastingBarFrame.startTime;
		end
	elseif (event == "SPELLCAST_STOP") then
		CastTime.delaySum = 0;
		CastTime.sign = "+";
		if (not CastingBarFrame.channeling) then
			CastTimeFrame:Hide();
			CastTimeText:SetText("");
			CastTimeMFText:SetText("");
		end
	else
		CastTime.delaySum = 0;
		CastTime.sign = "+";
		CastTimeFrame:Hide();
		CastTimeText:SetText("");
		CastTimeMFText:SetText("");
	end
end

function CastTime_OnUpdate()
	--Set the timer to the current time, max 4 chars. > 100 sec, seconds
	--  only; > 10 sec, tenths; < 10 sec hundredths. Max is present to
	--  prevent timer from showing negative if UI is slow on updating the hide.

	if (not CastingBarFrame:IsShown() or CastTime.display == 0) then
		CastTimeText:SetText("");
		CastTimeMFText:SetText("");
		CastTimeFrame:Hide();
		return;
	end

	local current_time = CastingBarFrame.maxValue - GetTime();
	if (CastingBarFrame.channeling) then
		current_time = CastingBarFrame.endTime - GetTime();
	end

	local subto = 4;
	if (current_time < 10 and CastTime.hun == 0) then
		subto = 3;
	end

	local text = string.sub(math.max(current_time,0)+0.001, 1, subto);
	if (CastTime.delaySum ~= 0 and CastTime.delay == 1) then
		local delay = string.sub(math.max(CastTime.delaySum/1000, 0)+0.001, 1, subto);

		if (CastingBarFrame.channeling == 1) then
			CastTime.sign = "-";
		end
		if (delay ~= "0.00") then
			text = "|cffcc0000"..CastTime.sign..delay.."|r "..text;
		end
	end
	if (CastTime.mf == 1) then
		CastTimeMFText:SetText(text);
	else
		CastTimeText:SetText(text);
	end
end
